@@ -206,7 +206,7 @@ typedef enum {
 	SS_ResyncRunning=-8,
 	SS_AlreadyStandAlone=-9,
 	SS_CW_FailedByPeer=-10,
-	SS_CanNotOutdateDL=-11,
+	SS_IsDiskLess=-11,
 	SS_DeviceInUse=-12
 } set_st_err_t;
 
